package io.skore.smooch_plugin.util

import android.annotation.SuppressLint
import android.app.Application
import android.content.Context
import android.util.Log
import io.flutter.plugin.common.MethodChannel
import io.skore.smooch_plugin.data.SmoochAuth
import io.skore.smooch_plugin.delegate.ConversationDelegate
import io.skore.smooch_plugin.delegate.MessageModifierDelegate
import io.smooch.core.InitializationStatus
import io.smooch.core.LoginResult
import io.smooch.core.Settings
import io.smooch.core.Smooch
import java.lang.ref.WeakReference

object SmoochUtil {
    private val tag = SmoochUtil::class.java.simpleName

    private const val SMOOCH_PREFS = "SMOOCH_PREFS"
    private const val AUTH_APP_ID = "AUTH_APP_ID"
    private const val AUTH_USER_ID = "AUTH_USER_ID"
    private const val AUTH_TOKEN = "AUTH_TOKEN"

    var methodChannelRef: WeakReference<MethodChannel?>? = null

    fun init(app: Application, appId: String, onFinish: ((Boolean) -> Unit)? = null) {
        val initStatus = Smooch.getInitializationStatus()
        if (initStatus != InitializationStatus.Success) {
            Smooch.init(app, Settings(appId)) { initResponse ->
                Log.d(tag, "Smooch init status: ${initResponse.status}")
                onFinish?.invoke(initResponse.status == 200)
            }
        } else {
            Log.d(tag, "Smooch already initialized.")
            onFinish?.invoke(true)
        }
    }

    fun login(app: Application, smoochAuth: SmoochAuth, onFinish: ((Boolean) -> Unit)? = null) = init(app, smoochAuth.appId) { isInit ->
        if (!isInit) {
            Log.d(tag, "Failed to initialize smooch. Login can not proceed.")
            onFinish?.invoke(false)
            return@init
        }

        val loginResult = Smooch.getLastLoginResult()
        if (loginResult != LoginResult.Success) {
            Smooch.login(smoochAuth.userId, smoochAuth.token) { loginResponse ->
                Log.d(tag, "Smooch login status: ${loginResponse.status}")
                onFinish?.invoke(loginResponse.status == 200)
            }
        } else {
            Log.d(tag, "User already logged in.")
            onFinish?.invoke(true)
        }
    }

    fun addDelegates() {
        Smooch.getConversation()?.apply {
            methodChannelRef?.get()?.let {
                delegate = ConversationDelegate(it)
            } ?: run {
                Log.e(tag, "Method channel is not available.")
            }
            messageModifierDelegate = MessageModifierDelegate()
        }
    }

    @SuppressLint("ApplySharedPref")
    fun setAuth(context: Context, smoochAuth: SmoochAuth?) {
        context.getSharedPreferences(SMOOCH_PREFS, Context.MODE_PRIVATE)?.run {
            edit()?.apply {
                putString(AUTH_APP_ID, smoochAuth?.appId)
                putString(AUTH_USER_ID, smoochAuth?.userId)
                putString(AUTH_TOKEN, smoochAuth?.token)
                commit()
            }
        }
    }

    @SuppressLint("ApplySharedPref")
    fun getAuth(context: Context) = context.getSharedPreferences(SMOOCH_PREFS, Context.MODE_PRIVATE)?.run {
        val appId = getString(AUTH_APP_ID, null)
        val userId = getString(AUTH_USER_ID, null)
        val token = getString(AUTH_TOKEN, null)
        if (appId.isNullOrBlank() || userId.isNullOrBlank() || token.isNullOrBlank()) null else SmoochAuth(appId, userId, token)
    }
}